# This file provides updated code for Ward and Gleditsch 2018. 
#   Spatial Regression Models, 2nd edition
#
# This code was updated from the original code 
# by Kristian Skrede Gleditsch and Andrine Fogstad.
# It uses Groundhog for version control
# 
# The original version of this code was largely written by
# by the late Michael D. Ward ca 2017. In some cases it relies on
# packages and technology that have changed substantially
# since first written, and some online data sources appear to 
# change over time as data are revised and updated. We have tried to 
# stick as closely as possible to the original code used for the book. 
# We note some differences in the below text

# Please direct all questions to ksg@essex.ac.uk
#
# This version: 2023-07-07
usedate <- as.Date("2023-07-07")-3 
# To prevent problems with time zones Groundhog needs date at least two days prior

# Set working directory
setwd("C:/Users/ksg/Dropbox/2023_SRMrepository/")


require(groundhog)

pckgs <- c("choroplethr","choroplethrAdmin1","classInt","countrycode","cshapes",
  "ggmap","ggplot2",
  "HistData",
  "maps","mapproj","maptools",
  "plotly","plyr",
  "RColorBrewer","rgdal","rgeos",
  "sp",
  "treemap",
  "wbstats","WDI")

groundhog.library(pckgs,usedate) #, force.install=TRUE can be used if rerunning




######
# Figure 2.2 
# Cholera in London, aka the John Snow map

# Load grayscale image
OSMap <- readGDAL("shapefiles/SnowGIS/OSMap_Grayscale.tif")

# Open graphics device, i.e., PDF, PNG etc.

# Set margins to zero
par(mar = c(0,0,0,0))

# Plot map
image(OSMap, col = grey(1:999/1000))

## Load geographical information of sites of cholera deaths
Cholera_Deaths <- readOGR("shapefiles/SnowGIS/","Cholera_Deaths")

# Plot sites of cholera deaths
plot(Cholera_Deaths, pch = 16, cex = log(Cholera_Deaths$Count), col = "red", add = TRUE)

## Load geographical information of sites of pumps
Pumps <- readOGR("shapefiles/SnowGIS/","Pumps")

## Plot sites of pumps

plot(Pumps, add = TRUE, pch = 25, col = "blue",cex=2)

###


###
# Figure 2.3
# Simple Latitude - Longitude Projection 

# Read shapefile
wmap <- readOGR(dsn="shapefiles/110m_cultural", layer="ne_110m_admin_0_countries")

# Plot map
plot(wmap)

# Add lines for slicing
abline(v=180,lwd=4)
abline(v=-180,lwd=4)
####


###
# Figure 2.4 
# World Map: Robinson Projection

# Read shapefile
wmap <- readOGR(dsn="shapefiles/110m_cultural", layer="ne_110m_admin_0_countries")
wmap  <- wmap [wmap@data$name!="Antarctica", ]  # Get Rid of Antarctica
wmap  <- wmap [wmap@data$name!="Greenland", ] # Get Rid of Greenland

# Transform projection to Robinson
wmap_robin <- spTransform(wmap, CRS("+proj=robin"))
wmap_df_robin <- fortify(wmap_robin) # for use with ggplot

# Plot map
ggplot(wmap_df_robin, aes(long,lat, group=group)) + 
  geom_polygon(fill="white") +  geom_path(color="black") +
  labs(title="World Map, with Robinson Projection",x="Longitude",y="Latitude") + 
  coord_equal() + scale_x_continuous(labels = waiver()) + scale_y_continuous(labels = waiver())
###


####
# Figure 2.5 Winkel Tripel Projection

# Read map
wmap <- readOGR(dsn="shapefiles/110m_cultural", layer="ne_110m_admin_0_countries")
wmap  <- wmap [wmap@data$name!="Antarctica", ]  # Get rid of Antarctica
wmap  <- wmap [wmap@data$name!="Greenland", ] # Get rid of Greenland

# Transform projection
wmap_wIII <- spTransform(wmap, CRS("+proj=wintri"))
wmap_df_wIII <- (wmap_wIII) # not fortified

# Plot
ggplot(wmap_df_wIII, aes(long,lat, group=group)) + 
  geom_polygon(fill="grey60") +  geom_path(color="black") +
  labs(title="World map, with Winkel Tripel projection",x="Longitude",y="Latitude") + 
  coord_equal() + scale_x_continuous(breaks=0) + scale_y_continuous(breaks=0)


###
# Figure 2.6 Winkel Tripel with graticules and bounding box

wmap <- readOGR(dsn="shapefiles/110m_cultural", layer="ne_110m_admin_0_countries")
wmap  <- wmap [wmap@data$name!="Antarctica", ]  # Get rid of Antarctica
wmap  <- wmap [wmap@data$name!="Greenland", ] # Get rid of Greenland

# Transform
wmap_wIII <- spTransform(wmap, CRS("+proj=wintri"))
wmap_df_wIII <- (wmap_wIII) # not fortified

# Read graticules and bounding box, reproject
grat <- readOGR("shapefiles/ne_110m_graticules_all", layer="ne_110m_graticules_15") 
grat_wintri <- spTransform(grat, CRS("+proj=wintri"))  # reproject graticule
grat_df_wintri <- (grat_wintri) #not fortifyied
bbox <- readOGR("shapefiles/ne_110m_graticules_all", layer="ne_110m_wgs84_bounding_box") 
bbox_wintri <- spTransform(bbox, CRS("+proj=wintri"))  # reproject bounding box
bbox_wintri_df <- (bbox_wintri) #not fortifyied

# Plot
ggplot(bbox_wintri_df, aes(long,lat, group=group)) + 
  geom_polygon(fill="white") +  geom_path(color="red") +
  geom_polygon(data=wmap_df_wIII, aes(long,lat, group=group,fill=id )) + 
  geom_path(data=grat_df_wintri, aes(long, lat, group=group, fill="NULL"), 
            linetype="solid", color="grey") +
  labs(title="World Map (Winkel Tripel)",x="Longitude",y="Latitude") + 
  theme(legend.position="none")
###

###
# Figure 2.7
# Japanese Population Density
# Note: this example now requires a Google API key
# You can do this at https://mapsplatform.google.com/
# You will need to provide a credit card for verification as of 2023-07-07
# 

# Map Japanese Population Density in 2012 over a google reference map.
data(admin1.regions)
data(df_japan_census)
df_japan_census$value <- df_japan_census$pop_density_km2_2010

admin1_choropleth("japan", df_japan_census,
                  title="Japanese Population Density \n per square kilometer\n 2012",
                  num_colors = 6,reference_map=T)
####


###
# Figure 2.8
# Air Pollution from World Bank. 2013 Data.

# A little experimentation may be necessary to find year and data 
wbi.df <- WDI(indicator = "EN.ATM.CO2E.KT" ,
         start = 2011, end = 2015, extra = TRUE)
wbi.df <- wbi.df[!(wbi.df$region %in% "Aggregates"),]
wbi.df <- wbi.df[!is.na(wbi.df$EN.ATM.CO2E.KT),]

pollution_data <- wb(indicator = "EN.ATM.PM25.MC.M3", startdate = 2013, enddate = 2014)
temp.data <- pollution_data

map <- cshp(date = as.Date("2015-01-01")) 
map <- as(map, "Spatial") # Convert simple feature object to spatial

data <- data.frame(id=rownames(map@data),gwcode=map@data$gwcode)

# Use countrycode package to convert from Gleditsch and Ward code to ISO2
data$iso2c   <- countrycode(data$gwcode, origin = "gwn", destination = "iso2c")
# Not matched unamiguously, added manually
# 347 Kosovo, 678 Yemen, 816 Vietnam
data$iso2c[data$gwcode==347] <- "XK"
data$iso2c[data$gwcode==678] <- "YE"
data$iso2c[data$gwcode==816] <- "VN"

data$id <-as.character(data$id)
data <- merge(data,temp.data,by="iso2c",all.x=TRUE)

map.df <-fortify(map)
map.df <-join(map.df,data,by="id")

ggplot(map.df,aes(long,lat, group=group)) + geom_polygon(aes(fill=value)) + scale_fill_gradient(low="red",high="blue",
                    na.value = "grey80",guide= "colourbar", 
                    space ="Lab") + 
labs(
  title="2013 Air pollution from World Bank Indicators \n Mean Annual Exposure (micrograms per cubic meter)",
  x="",y="") + 
theme(axis.text=element_blank(),axis.ticks=element_blank())
###


###
# Figure 2.9 Conflict Events in Bosnia-Herzegovina
# Figure 2.10 Aggregations of Conflict events in B-H
# Figure 2.11 Conflict Events in B-H with Elevation

# Read municipalities shapefile
units <- readOGR("shapefiles/bosnia/","bosnia_units")

units$unit <- c(1:dim(units@data)[1])

# Read the ACLED conflict events
events <- readOGR("shapefiles/bosnia","bosnia_twosided") 

# Overlay analysis (new preferred syntax)
res.overlay <- over(events, geometry(units), returnList = F)

# Add a unit index number to events
events$unit <- res.overlay

# Find the ID of the events outside units
events[is.na(events$unit),]

# Tag all events outside units
events$inside <- 1
events$inside[is.na(events$unit)] <- 0

# Delete events without a unit index number
events2 <- events[!is.na(events$unit),]

# Plot all events and municipalities
plot(units)
plot(events, add=T, cex=0.5, pch=19)

# Aggregate by unit index
events2$event <- 1
events.by.unit <- aggregate(event ~ unit, data=events2, sum)
names(events.by.unit)[2] <- "event.count"

# Add to units spatial data frame
units@data = data.frame(units@data, 
                        events.by.unit[match(units@data[,"unit"], events.by.unit[,"unit"]),])
units$event.count[is.na(units$event.count)] <- 0

# Plot event counts
breaks <- c(0,1,5,10,20,max(units$event.count))
cols <- brewer.pal(length(breaks), "Greys")
plot(units, bty="n", col=cols[findInterval(units$event.count, 
                                           breaks, all.inside=T)])
legend(x="bottomleft", legend = leglabs(breaks), fill=cols, 
       bty="n", title="Conflict events")

# Load terrain information
terrain <- readAsciiGrid("shapefiles/bosnia/bosnia_terrain.asc", proj4string=CRS("+proj=longlat +ellps=WGS84"))
summary(terrain)

# The complete plot of municipalities over terrain
image(terrain, col=rev(gray.colors(12)))

plot(units, add=T)

# Superimpose the conflict events
plot(events2, add=T, pch=19, cex=0.5)
###
