# This file provides updated code for Ward and Gleditsch 2018. 
#   Spatial Regression Models, 2nd edition
#
# This code was updated from the original code 
# by Kristian Skrede Gleditsch and Andrine Fogstad.
# It uses Groundhog for version control
# 
# The original version of this code was largely written by
# by the late Michael D. Ward ca 2017. In some cases it relies on
# packages and technology that have  changed substantially
# since first written, and some online data sources appear to 
# change over time as data are revised and updated. We have tried to 
# stick as closely as possible to the original code used for the book. 
# We note some differences in the below text
#
# Please direct all questions to ksg@essex.ac.uk
#
# This version: 2023-07-20
usedate <- as.Date("2023-07-07")-3 
# To prevent problems with time zones Groundhog needs date at least two days prior

# Set working directory
setwd("C:/Users/ksg/Dropbox/2023_SRMrepository/")


require(groundhog)

pckgs <- c("choroplethr","choroplethrAdmin1","classInt","countrycode","cshapes",
           "EnvStats",
           "foreign",
           "GGally","ggmap","ggplot2","gridExtra","grid",
           "HistData",
           "igraph",
           "maps","mapproj","maptools",
           "network",
            "plotly","plyr",
           "RColorBrewer","rgdal","rgeos",
           "sp","sna",
           "treemap",
           "wbstats","WDI",
           "xtable")

groundhog.library(pckgs,usedate)#, force.install=TRUE)


### 
# This section prepares the data shown in Table 3.1


# Read P5 data
p5 <- read.csv("rawdata/p5.csv") # This file for some reasons has to US observations
p5[p5$scode %in% "USA",]
p5 <- unique(p5)

# Extract map from Cshapes 
wmap <- cshp(date = as.Date("2015-01-01"),useGW=TRUE)  # get map from CSHAPES

wmap <- as(wmap, "Spatial") # Convert simple feature object to spatial

wmap@data$id <- as.character(rownames(wmap@data))
wmap.df <- wmap@data

# Note: the p5.csv file seems to use ISO3 labels and not polity acronyms
wmap.df$scode   <- countrycode(wmap.df$gwcode, origin = "gwn", destination = "iso3c") 
wmap.df[is.na(wmap.df$scode),"country_name"] # Check missing
wmap.df$scode[grep("Vietnam",wmap.df$country_name)] <- "VNM" 
wmap.df$scode[grep("Kosovo",wmap.df$country_name)] <- "RKS" 
wmap.df$scode[grep("Yemen",wmap.df$country_name)] <- "YEM" 
setdiff(p5$scode,wmap.df$scode)

# Add on to wmap.df
wmap.df <- join(wmap.df,p5[,c("scode","democ","autoc")],by="scode",type="left")

# Extract GDP data from World Bank
library(wbstats)
gdp <- wb(indicator = c("NY.GDP.PCAP.CD"),
          startdate = 2014, enddate = 2014,country="countries_only")
gdp$lngdpvalue <- log(gdp$value)

gdp$gwcode <- countrycode(gdp$iso3c, origin = "iso3c", destination = "gwn")
gdp$country[is.na(gdp$gwcode)] # Check missing, all but Kosovo and Yemen missing in Polity
gdp$gwcode[grep("Kosovo",gdp$country)] <- 347
gdp$gwcode[grep("Yemen",gdp$country)] <- 678

# join the GDP data to the map data.frame
wmap.df <- join(wmap.df,gdp[,c("gwcode","lngdpvalue")],by="gwcode",type="left")

# create polity score
wmap.df$polity <- wmap.df$democ - wmap.df$autoc

# Data for Table 3.1, show top 10 and bottom 10
dat <- wmap.df[!is.na(wmap.df$lngdpvalue) & !is.na(wmap.df$polity),]

# top
dat[rev(order(dat$lngdpvalue)),c("country_name","lngdpvalue","polity")][1:10,]
# bottom 10
dat[order(dat$lngdpvalue),c("country_name","lngdpvalue","polity")][1:10,]

####
# Code for Table 3.2, OLS output

# Note that these figures differ from those reported in the book,
# possibly due to differences in input WB data/code alignment, but the conclusion
# from the analysis remains the same

ols.1.2 <- glm(wmap.df$polity ~ wmap.df$lngdpvalue)
summary(ols.1.2)
xtable(ols.1.2)
###


### 
# This code recreates Figure 3.1
dat <- subset(wmap.df,select = c("polity","lngdpvalue"))
ggplot(dat, aes(x=lngdpvalue, y=polity)) +
  geom_point(shape=16) +    
  geom_smooth(method=lm) + 
  labs(x = "GDP, logged per capita",y = "Polity Score") +
  coord_fixed(.2) +
  ggtitle("Figure 3.1") 
###

###
# This code recreates Figure 3.2
qqPlot(residuals(ols.1.2),pch=19,las = 1,cex=.7,
       ylab="Sample Quantiles",xlab="Theoretical Quantiles",
       add.line = T, line.col = "grey", line.lwd = 3,
       main="Figure 3.2")
###

###
# This code recreates Figure 3.3 
plot(density(residuals(ols.1.2)),ylim=c(0,.5),
     xlim=c(-20,10),bty="n",axes=F,xlab="",ylab="",
     lwd=3,main="Figure 3.3")
lines(density(rnorm(100000,0,1)),lwd=3,col="gray")
axis(side=1,at=c(-20,-10,0,10))
abline(v=0,lwd=3, col="black")
###


# Table 3.3 is not replicated.
# Table 3.4 is not replicated


### 
# This code recreates Figure 3.4 
# Network Visualization among eight European countries

# Extract map for eight counties listed by numeric GW country code 
easyeight.shp <- wmap[wmap@data$gwcode %in% c(200,220,260,325,375,380,385,390),]

# Create a distance matrix from Cshapes
wmat.195 <- distmatrix(as.Date("2015-1-1"), type="mindist") # This takes a fair amount of time

# Subset matrix to the list of eight countries above
ee <- as.character(c(200,220,260,325,375,380,385,390))
wmat.8 <- wmat.195[ee,ee]

# Check if each dyadic element within 200km
w8.dist <- wmat.8
w8.dist[w8.dist<=200] <- 1
w8.dist[w8.dist>200] <- 0

# Rename col/rows to acronyms
colnames(w8.dist) <- rownames(w8.dist) <-c("UKG","FRN","ITA","GER","DEN","SWE","NOR","FIN")

# Set diagonal to 1
diag(w8.dist) <- 0

net <- network(w8.dist)

# vertex names
set.seed(12345)
network.vertex.names(net) = c("UKG","FRN","ITA","GER","DEN","SWE","NOR","FIN")

# Plotting figure 3.4
ggnet2(net, node.size = 14, node.color = "gray30", edge.size = 1, edge.color = "grey",
       mode = "kamadakawai",label = TRUE,color = "black", label.color = "white")
###


###
# This recreates a version similar to Figure 3.5  
# Simple Network Visualizaiton of Linkages among 195 Countries

wmat.bin <- wmat.195
wmat.bin[wmat.195<=200] <- 1
wmat.bin[wmat.195>200] <- 0
diag(wmat.bin) <- 0

# Check order is correct
table(wmap.df$gwcode==row.names(wmat.bin)) # All sorted

# Add acronyms
wmap.df$gwacr <- countrycode(wmap.df$gwcode, origin = "gwn", destination = "gwc")
wmap.df$gwacr[wmap.df$gwcode==816] <- "DRV"
colnames(wmat.bin) <- rownames(wmat.bin) <- wmap.df$gwacr

# Convert to an ipgraph 
net.igraph <- graph_from_adjacency_matrix(wmat.bin, 
                                          mode = c("undirected"),  diag = FALSE)
sizes <- wmap.df$lngdpvalue
sizes[is.na(sizes)] <- 5.75

V(net.igraph)$lngdpvalue <- sizes*1.5
# Set size proportional to outdegree
V(net.igraph)$size <- igraph::degree(net.igraph) # Avoid confusion with sna degree

# Find all isolates
iso <- V(net.igraph)[igraph::degree(net.igraph)==0]
# Remove isolates from the graph
g2 <- igraph::delete.vertices(net.igraph, iso)


# Plot version with all countries, including isolates

plot(net.igraph,vertex.label.cex=.6,
     vertex.color = "skyblue")

# Plot version without isolates

plot(g2,vertex.label.cex=.6,
     vertex.color = "skyblue")

####


# Table 3.5 is not reproduced 

# Table 3.6 is not reproduced Top and Bottom 10 Spatial Lag Polity Scores


####
# Figure 3.6 Map of Centroids

# Read a Robinson projection map from an ESRI shapefile
rob.shp <- readOGR(dsn="shapefiles/world2002",layer="wg2002worldmap")

# Add coordinates
coords <- coordinates(rob.shp)

# Plot the map, without bounding box
plot(rob.shp, border="Grey",
     xaxt="n", yaxt="n", bty="n",
     lwd=.75, las=1, ylab="",
     main="Centroids and Capitals", xlab="")

# Add the centroids in red
points(coordinates(rob.shp), pch=19,cex=.25, col="red")

# Latitude and longitude of capitals
ll.mat <- cbind(as.numeric(as.character(rob.shp$Long)),
                as.numeric(as.character(rob.shp$Lat)))
row.names(ll.mat) <- 1:nrow(ll.mat)
llCRS <- CRS("+proj=longlat +ellps+WGS84")
longlat <-  SpatialPoints(ll.mat,proj4string = llCRS)

longlat.transf <- spTransform(longlat,
                              CRS("+proj=robin +ellps+WGS84+lon_0=0 + x_0=0 +y_0=0"))
points(coordinates(longlat.transf),pch=19,cex=.25,col="black")

# Add segments between centroids and capitals
# Kiribati crosses date line
# Note that Kiribati is element 92 since row.names in rob.shp named from 0
diffs <- cbind(coordinates(rob.shp),coordinates(longlat.transf))
segments(diffs[-92,1],diffs[-92,2],diffs[-92,3],diffs[-92,4],col="slategray4")
####

####
# Figure 3.7 Anselin-Moran Plot of OLS Residuals 
#
# Prune Data Set
moran.df <- subset(wmap.df,select = c("gwcode","polity","lngdpvalue"))
pruned <- na.omit(moran.df)
pruned <- pruned[with(pruned, order(gwcode)), ]
row.names(pruned) <- pruned$gwcode

# wdist has the distance scores, in order of GW codes
wmap.dist <- wmat.195
index <-as.character(pruned$gwcode)
w.dist <- wmap.dist[index,index] # subset

# recode distance data to be binary, for distance < 550km
w.dist[w.dist<=400] <- 1
w.dist[w.dist>=400] <- 0
w.dist.rowstd <- w.dist/rowSums(w.dist)

# create spatial lag of residuals
resid.splag <- w.dist.rowstd%*%resid(ols.1.2)
resid.splag.std <- scale(resid.splag)
resid.std <- scale(resid(ols.1.2))

# Plot
plot(resid.std,resid.splag.std,las=1,pch=19,xlim=c(-3,3),
     ylim=c(-3,3),ylab="Standardized Spatial Lag of OLS Residuals",
     xlab="Standardized OLS Residuals",main="")
polygon(c(0,0,1,1),c(0,1,1,0), col=rgb(	135/255,206/255,250/255,0.5))
polygon(c(-1,-1,0,0),c(-1,0,0,-1), col=rgb(	135/255,206/255,250/255,0.5))
points(resid.std,resid.splag.std,pch=19)

segments(-2, -2, -2, 2,lwd=2)
segments(2, -2, 2, 2, lwd=2)
segments(-2, 2, 2,2, lwd=2)
segments(-2, -2, 2,-2, lwd=2)
segments(-1, -1, -1, 1, lwd=2)
segments(1, -1, 1, 1, lwd=2)
segments(-1, 1, 1,1, lwd=2)
segments(-1, -1, 1,-1, lwd=2)
segments(-2, 0, 2,0, lwd=2)
segments(0, -2,0,2, lwd=2)

rug(resid.std, ticksize = 0.03, side = 1, lwd = 0.5)
rug(resid.splag.std, ticksize = 0.03, side = 2, lwd = 0.5)

abline(lm(resid.splag.std ~ -1 + resid.std),lwd=2)
###

