# This file provides updated code for Ward and Gleditsch 2018. 
#   Spatial Regression Models, 2nd edition
#
# This code was updated from the original code 
# by Kristian Skrede Gleditsch and Andrine Fogstad.
# It uses Groundhog for version control
# 
# The original version of this code was largely written by
# by the late Michael D. Ward ca 2017. In some cases it relies on
# packages and technology that have changed substantially
# since first written, and some online data sources appear to 
# change over time as data are revised and updated. We have tried to 
# stick as closely as possible to the original code used for the book. 
# We note some differences in the below text
# Please direct all questions to ksg@essex.ac.uk
#
# This version: 2023-07-20
usedate <- as.Date("2023-07-07")-3 
# To prevent problems with time zones Groundhog needs date at least two days prior

# Set working directory
setwd("C:/Users/ksg/Dropbox/2023_SRMrepository/")


require(groundhog)

pckgs <- c("choroplethr","choroplethrAdmin1","classInt","countrycode","cshapes",
           "EnvStats",
           "foreign",
           "GGally","ggmap","ggplot2","gridExtra","grid",
           "HistData",
           "igraph",
           "maps","mapproj","maptools",
           "network",
           "plotly","plyr",
           "RColorBrewer","rgdal","rgeos",
           "sp","spatialreg","spdep","sna",
           "treemap",
           "wbstats","WDI",
           "xtable")

groundhog.library(pckgs,usedate)#, force.install=TRUE)


###
# Table 4.1 
# Note that this data does not match exactly what is reported 
# and has a higher N than the the table in the book. However, 
# the reported results are substantively similar.

# Read P5 data
p5 <- read.csv("rawdata/p5.csv") # This file for some reasons has to US observations
p5[p5$scode %in% "USA",]
p5 <- unique(p5)

# Extract map from Cshapes 
wmap <- cshp(date = as.Date("2015-01-01"),useGW=TRUE)  # get map from CSHAPES

wmap <- as(wmap, "Spatial") # Convert simple feature object to spatial

wmap@data$id <- as.character(rownames(wmap@data))
wmap.df <- wmap@data

# Note: the p5.csv file seems to use ISO3 labels and not polity acronyms
wmap.df$scode   <- countrycode(wmap.df$gwcode, origin = "gwn", destination = "iso3c") 
wmap.df[is.na(wmap.df$scode),"country_name"] # Check missing
wmap.df$scode[grep("Vietnam",wmap.df$country_name)] <- "VNM" 
wmap.df$scode[grep("Kosovo",wmap.df$country_name)] <- "RKS" 
wmap.df$scode[grep("Yemen",wmap.df$country_name)] <- "YEM" 
setdiff(p5$scode,wmap.df$scode)

# Add on to wmap.df
wmap.df <- join(wmap.df,p5[,c("scode","democ","autoc")],by="scode",type="left")

# Extract GDP data from World Bank
library(wbstats)
gdp <- wb(indicator = c("NY.GDP.PCAP.CD"),
          startdate = 2014, enddate = 2014,country="countries_only")
gdp$lngdpvalue <- log(gdp$value)

gdp$gwcode <- countrycode(gdp$iso3c, origin = "iso3c", destination = "gwn")
gdp$country[is.na(gdp$gwcode)] # Check missing, all but Kosovo and Yemen missing in Polity
gdp$gwcode[grep("Kosovo",gdp$country)] <- 347
gdp$gwcode[grep("Yemen",gdp$country)] <- 678

# join the GDP data to the map data.frame
wmap.df <- join(wmap.df,gdp[,c("gwcode","lngdpvalue")],by="gwcode",type="left")

# create polity score
wmap.df$polity <- wmap.df$democ - wmap.df$autoc

# Join polity data onto wap
wmap@data$polity <- wmap.df$polity
# for convenience assign 0 to NA
wmap@data$polity[is.na(wmap@data$polity)]<- 0

# join the GDP data to the map data.frame
wmap@data$lngdpvalue <- wmap.df$lngdpvalue
# Add on missing values hacked from the interwebs
wmap@data[is.na(wmap@data$lngdpvalue),"country_name"]
#[1] "Eritrea"                         "Taiwan"                          "Korea, People's Republic of"    
#[4] "Vietnam, Democratic Republic of"
wmap@data$lngdpvalue[wmap@data$country_name=="Eritrea"] <- 6.36647
wmap@data$lngdpvalue[wmap@data$country_name=="Korea, People's Republic of"] <-6.920672
wmap@data$lngdpvalue[wmap@data$country_name=="Taiwan"] <- 10.02398
wmap@data$lngdpvalue[wmap@data$country_name=="Vietnam, Democratic Republic of"] <-  8.887473

# Create a distance matrix from Cshapes
wmat.195 <- distmatrix(as.Date("2015-1-1"), type="mindist") # This takes a fair amount of time

# Recode distance data to be binary, for distance < 400km
w.dist <- wmat.195
w.dist[w.dist<=400] <- 1
w.dist[w.dist>400] <- 0
# 0 on diagonal
diag(w.dist) <- 0

# row standardize
w.dist.rstd <- w.dist/rowSums(w.dist)

# get rid of isolates
omits <- which(rowSums(w.dist)==0)

# create spatial lag of polity. round result
wmap@data$sp.lag.polity <- round((w.dist.rstd)%*%(wmap@data$polity))


# wmap dataframe is sorted on gwcode 
# get rid of isolates

map.df <- wmap@data[-omits,]
w.d.rstd <- w.dist.rstd[-omits,-omits]

# Create a SP listw object of neighbors
listw.obj <- mat2listw(w.d.rstd, style = "W")

ols.fit <- glm(polity ~ lngdpvalue, data=map.df)

moran.test(resid(ols.fit),listw.obj)

sar.fit <- lagsarlm(polity ~ lngdpvalue, data=map.df, listw.obj,type="lag",
                  method="eigen")

summary(ols.fit)
summary(sar.fit)
###

# Table 4.2 not reproduced

###
# Figure 4.2

# Set up matrix for predicted values given Polity x GDP per capita
pry.mat <- matrix(NA,nrow=21,ncol=length(seq(5.750,12.100,0.5)))
lngdppc <- seq(5.750,12.100,0.5)
gdppc <- exp(seq(5.750,12.100,0.5))
polity <- c(-10:10)

for(i in 1:dim(pry.mat)[1]){
  for(j in 1:dim(pry.mat)[2]){
    pry.mat[i,j] <- c(summary(sar.fit)$Coef[1,1]+
                        summary(sar.fit)$rho*polity[i]+
                        summary(sar.fit)$Coef[2,1]*lngdppc[j])
  }
}

# Make matrix list for plotting
# Use GDP pc rather than ln GDP pc scale

prymat2.pl <- list(polity,gdppc,pry.mat)
names(prymat2.pl) <- c("x","y","z")

options(par(las=1,cex=1))
contour(prymat2.pl,nlevels=21,
        drawlabels=T,method="edge",axes=F,
        xlab="Democracy among neighbors",
        ylab="GDP per capita")
axis(1)
axis(2, at = 
       c(0,10000,20000,30000,40000,50000,60000,70000,80000,90000,100000,120000),
     labels = c("0","10K" 
                ,"20K","30K","40K","50K","60K","70K","80K","90K","100K","120K"),tick = TRUE,
     lty = "solid", lwd = 1, col = NULL)
###

###
# Figure 4.3
#

# Read data
shin <- read.csv("rawdata/ITALY/italyturnout.csv", header=T)

# Read Italy shape file
italy.shp <- readOGR(dsn="shapefiles/ITALY", layer="turnout")
plot(italy.shp)

# Create interval breaks 
breaks <-round(quantile((shin$turnout), seq(0,1,1/4), na.rm=TRUE),1)

# Select greyscale scheme
cols <- brewer.pal(length(breaks), "Greys")

# Plot 
plot(italy.shp,bty="n", border="slategray3", 
     xaxt="n",yaxt="n",lwd=.5,las=1,ylab="", main="", xlab="")
plot(italy.shp,bty="n",
     col=cols[findInterval(shin$turnout, breaks, all.inside=T)], add=T)

# Add legend
leg.txt <- c(">90", "87-90",
             "83-86", "79-82","59-78")
legend("topright",y=NULL, leg.txt, fill = rev(cols),
       angle = 45, density = NULL, bty = "n",
       cex = 1,
       xjust = 0, yjust = 1, x.intersp = 1, y.intersp = 1,
       adj = c(0, 0.5),
       plot = TRUE,  horiz = F, title = "% Turnout",
       inset = 0)
###


### 
# Code for Moran's I tests reported
# Extract neighbors at 50 km
dnn50km <- dnearneigh(coordinates(italy.shp), 0, 50000)

# Moran's I tests
moran.test(shin$gdpcap,nb2listw(dnn50km))
moran.test(shin$turnout,nb2listw(dnn50km))
moran.test(shin$gdpcap,nb2listw(dnn50km),randomisation=F)
moran.test(shin$turnout,nb2listw(dnn50km),randomisation=F)
###

### 
# Table 4.4
ols.fit <- glm(turnout ~ log(gdpcap), data=shin)
summary(ols.fit)
###

###
# Table 4.5

sar.fit <- lagsarlm(turnout ~ log(gdpcap), data=shin,
                     nb2listw(dnn50km), method="eigen", quiet=TRUE)
summary(sar.fit)
###


###
# Code for Figure 4.4

sldv.fit <- lagsarlm(turnout ~ log(gdpcap), data=shin,
                     nb2listw(dnn50km), method="eigen", quiet=TRUE)
wmat <- nb2mat(dnn50km,style="W")

# Get the rho
rho <- coef(sldv.fit)[1]
beta <- coef(sldv.fit)[2:3]
X <- cbind(1,log(shin$gdpcap))
Xs <- X
Xs[432] <- log(35)
I <- diag(length(shin$gdpcap))
Ey <- solve(I - rho*wmat)%*%(X%*%beta)
EyS <- solve(I - rho*wmat)%*%(Xs%*%beta)
dif <- EyS-Ey
 
effected <- cbind(as.character(shin$collegio),as.numeric(round(dif,2)))
voters <- subset(effected,effected[,2]>0)
ord <- order(voters[,2])
big <- voters[ord,]

# Effect plot

dif <- round(dif,0)
dif[dif<1] <- 0
breaks <- as.vector(c(0,1,6,17))
cols <- cbind("white","gray60","gray50","gray40")

plot(italy.shp,bty="n", border="gray85" ,xaxt="n",yaxt="n",lwd=.003,las=1,ylab="",main="",xlab="")
plot(italy.shp,bty="n", border="gray85" ,
    col=cols[findInterval(dif, breaks, all.inside=T)],add=T)
 
leg.text <- c(">17", ">6-17",
             ">1-6", "0-1")
      legend("topright",y=NULL, leg.text, fill = rev(cols),
             angle = 45, density = NULL, bty = "n",
              cex = 1,
             xjust = 0, yjust = 1, x.intersp = 1, y.intersp = 1,
             adj = c(0, 0.5),
             plot = TRUE,  horiz = F, title = "% Turnout Increase",
             inset = 0)

# Circle
it <- seq(0,2*pi, l=300)
r <- 2 +150000# radius
xt1 <- r*cos(it)+1069677
yt1 <- r*sin(it)+4235858
lines(xt1,yt1,lwd=2,lty=2)


### 
# Figure 4.5 

# Read shapefile lower 48 states 
us48polys <- readOGR(dsn="shapefiles/usa", layer="48_states")

# Extract centroids and neighbor matrix
centroids <- coordinates(us48polys)
us48.nb <- poly2nb(us48polys)
us48.listw <- nb2listw(us48.nb, style = "B")
us48.mat <- (nb2mat(us48.nb,style="B"))

colnames(us48.mat) <- rownames(us48.mat) <- us48polys$STATE_ABBR

usa <- network(us48.mat, directed=F)

set.seed(123)
# plot network first; then add state boundaries
plot.network(usa, displayisolates=T, displaylabels=F,
               boxed.labels=F, coord=centroids, label.col="gray20",
              usearrows=F, edge.col=rep("gray60",190),
               vertex.col="gray30",edge.lty=1)
plot(us48polys,add=T)
###


###
# Figure 4.6

# read in 2004 presidential votes
presvote <- read.csv("rawdata/2004presvote.csv", na.strings = "-", 
  header=TRUE, 
  colClasses = c("character","numeric","numeric","numeric","numeric","numeric","numeric"))

# expand abbreviation of D.C. 
presvote$State[9] <- "District of Columbia"

# Alaska and Hawaii are set aside
presvote <- presvote[-c(2,12),]

# Calculate vote totals and then join  with map data
# Set the NAs in the file to 0 for zero votes
presvote[is.na(presvote)] <- 0

usa.shp <- us48polys
names(presvote)[names(presvote)=="State"] <- "STATE_NAME"

# Preserve plot order
usaall <- merge(usa.shp, presvote,
                by = "STATE_NAME", 
                sort = F)

# now plot the Bush:Kerry vote ratio
usaall$bk <- usaall$Bush/usaall$Kerry
breaks <- round(quantile((bk), seq(0,1,1/5), na.rm=TRUE),1)                        
cols <- brewer.pal(length(breaks), "Greys") 

plot(us48polys, bty="n", border="slategray3", 
     xaxt="n", yaxt="n",
     lwd=.000000000125,las=1,ylab="",xlab="")
plot(us48polys, bty="n",
     col=cols[findInterval(bk, breaks, all.inside=T)], add=T)
legend(x = c(-125, -115), y = c(27, 32), legend = leglabs(breaks),fill = cols, bty = "n")
####

###
# Table 4.6

knn1 <- knn2nb(knearneigh(coordinates(usa.shp),k=4))
knn2 <- knn2nb(knearneigh(coordinates(usa.shp),k=12))

moran.test(bk, us48.listw)
moran.test(bk, nb2listw(knn1),random=T)
moran.test(bk, nb2listw(knn2),random=T)

geary.test(bk, us48.listw)
geary.test(bk, nb2listw(knn1),random=T)
geary.test(bk, nb2listw(knn2),random=T)
###


### 
# Table 4.7


# now read in the GSP Data
gsp <- read.csv("rawdata/stategsp.csv", na.strings = "-", header=TRUE,
                colClasses = c("character","numeric"))
names(gsp)[1] <- "STATE_NAME"

# Remove Alaska and Hawai'i
gsp <- gsp[-c(46,48),]
names(gsp)[1] <- "STATE_NAME"

us48dat <- join(as.data.frame(usaall), gsp, by="STATE_NAME", type="left")

centroids <- coordinates(us48polys)
us48.nb <- poly2nb(us48polys)
us48.listw <- nb2listw(us48.nb, style = "B")

bk.lm <-lm(bk ~ gpdgrowth, data = us48dat)

lm.morantest(bk.lm,us48.listw)
lm.morantest(bk.lm,nb2listw(knn1))
lm.morantest(bk.lm,nb2listw(knn2))

# Row Standardized Weights
us48.w <- nb2listw(us48.nb, style = "W")
us48.kw <- nb2listw(knn1,style="W")

# spatial regression

sldv.fit <- lagsarlm(bk ~ gpdgrowth, data = us48dat, listw=us48.listw)
sldv2.fit <-lagsarlm(bk ~ gpdgrowth, data = us48dat, listw=us48.kw)

summary(sldv.fit)
summary(sldv2.fit)
###


###
# Figure 4.7 

# Calculate the equilibrium effect
# ( I - \rho {W} )^{-1}\beta

xsim <- as.vector(us48dat$gpdgrowth)

xbeta <- xsim * sldv.fit$coefficients[2]
wmat <- nb2mat(us48.nb)
a <- solve(diag(length(xsim)) - sldv.fit$rho*wmat) %*% (xbeta)
round(a, 1)

# calculate the equilibrium effect
# ( I - \rho {W} )^{-1}\beta

xsim <- as.vector(us48dat$gpdgrowth)

xbeta <- xsim * sldv.fit$coefficients[2]
wmat2 <- nb2mat(knn1)
b <- solve(diag(length(xsim)) - sldv2.fit$rho*wmat2) %*% (xbeta)
round(b,1)

# Compare equilibrium effects
plot(density(a),bty="n",ylab="",lty=1,lwd=2,col="slategray",
     xlim=c(-0.6,0.1),
     ylim=c(0,8),
     xlab="Equilibrium Effects",main="")
lines(density(b),lty=1,lwd=2,col="black")
leg.txt <- c("Queen's Contiguity","4 Nearest Neighbors")
leg.col <- c("gray","slategray")
legend("topleft",y=NULL, leg.txt, leg.col, 
       angle = 45, density = NULL, bty = "n",
       cex = 1,
       xjust = 0, yjust = 1, x.intersp = 1, y.intersp = 1,
       adj = c(0, 0.5), 
       plot = TRUE,  horiz = F, title = "Weight Matrix",
       inset = 0)
###



