# This file provides updated code for Ward and Gleditsch 2018. 
#   Spatial Regression Models, 2nd edition
#
# This code was updated from the original code 
# by Kristian Skrede Gleditsch and Andrine Fogstad.
# It uses Groundhog for version control
# 
# The original version of this code was largely written by
# by the late Michael D. Ward ca 2017. In some cases it relies on
# packages and technology that have  changed substantially
# since first written, and some online data sources appear to 
# change over time as data are revised and updated. We have tried to 
# stick as closely as possible to the original code used for the book. 
# We note some differences in the below text
# Please direct all questions to ksg@essex.ac.uk
#
# This version: 2023-08-07
usedate <- as.Date("2023-07-07")-3 
# To prevent problems with time zones Groundhog needs date at least two days prior

# Set working directory
setwd("C:/Users/ksg/Dropbox/2023_SRMrepository/")


require(groundhog)

pckgs <- c("choroplethr","choroplethrAdmin1","classInt","countrycode","cshapes",
           "EnvStats",
           "foreign",
           "GGally","ggmap","ggplot2","gridExtra","grid",
           "HistData",
           "igraph",
           "maps","mapproj","maptools",
           "network",
           "plotly","plyr",
           "RColorBrewer","rgdal","rgeos",
           "sp","spatialreg","spdep","sna",
           "treemap",
           "wbstats","WDI",
           "xtable")

groundhog.library(pckgs,usedate)#, force.install=TRUE)


###
# Table 5.1 
# Note that this data does not match exactly what is reported
# in the book. However, the reported results are substantively similar.

# Read P5 data
p5 <- read.csv("rawdata/p5.csv") # This file for some reasons has to US observations
p5[p5$scode %in% "USA",]
p5 <- unique(p5)

# Extract map from Cshapes 
wmap <- cshp(date = as.Date("2015-01-01"),useGW=TRUE)  # get map from CSHAPES

wmap <- as(wmap, "Spatial") # Convert simple feature object to spatial

wmap@data$id <- as.character(rownames(wmap@data))
wmap.df <- wmap@data

# Note: the p5.csv file seems to use ISO3 labels and not polity acronyms
wmap.df$scode   <- countrycode(wmap.df$gwcode, origin = "gwn", destination = "iso3c") 
wmap.df[is.na(wmap.df$scode),"country_name"] # Check missing
wmap.df$scode[grep("Vietnam",wmap.df$country_name)] <- "VNM" 
wmap.df$scode[grep("Kosovo",wmap.df$country_name)] <- "RKS" 
wmap.df$scode[grep("Yemen",wmap.df$country_name)] <- "YEM" 
setdiff(p5$scode,wmap.df$scode)

# Add on to wmap.df
wmap.df <- join(wmap.df,p5[,c("scode","democ","autoc")],by="scode",type="left")

# Extract GDP data from World Bank
library(wbstats)
gdp <- wb(indicator = c("NY.GDP.PCAP.CD"),
          startdate = 2014, enddate = 2014,country="countries_only")
gdp$lngdpvalue <- log(gdp$value)

gdp$gwcode <- countrycode(gdp$iso3c, origin = "iso3c", destination = "gwn")
gdp$country[is.na(gdp$gwcode)] # Check missing, all but Kosovo and Yemen missing in Polity
gdp$gwcode[grep("Kosovo",gdp$country)] <- 347
gdp$gwcode[grep("Yemen",gdp$country)] <- 678

# join the GDP data to the map data.frame
wmap.df <- join(wmap.df,gdp[,c("gwcode","lngdpvalue")],by="gwcode",type="left")

# create polity score
wmap.df$polity <- wmap.df$democ - wmap.df$autoc

# Join polity data onto wap
wmap@data$polity <- wmap.df$polity
# for convenience assign 0 to NA
wmap@data$polity[is.na(wmap@data$polity)]<- 0

# join the GDP data to the map data.frame
wmap@data$lngdpvalue <- wmap.df$lngdpvalue
# Add on missing values hacked from the interwebs
wmap@data[is.na(wmap@data$lngdpvalue),"country_name"]
#[1] "Eritrea"                         "Taiwan"                          "Korea, People's Republic of"    
#[4] "Vietnam, Democratic Republic of"
wmap@data$lngdpvalue[wmap@data$country_name=="Eritrea"] <- 6.36647
wmap@data$lngdpvalue[wmap@data$country_name=="Korea, People's Republic of"] <-6.920672
wmap@data$lngdpvalue[wmap@data$country_name=="Taiwan"] <- 10.02398
wmap@data$lngdpvalue[wmap@data$country_name=="Vietnam, Democratic Republic of"] <-  8.887473

# Create a distance matrix from Cshapes
wmat.195 <- distmatrix(as.Date("2015-1-1"), type="mindist") # This takes a fair amount of time

# Recode distance data to be binary, for distance < 400km
w.dist <- wmat.195
w.dist[w.dist<=400] <- 1
w.dist[w.dist>400] <- 0
# 0 on diagonal
diag(w.dist) <- 0

# row standardize
w.dist.rstd <- w.dist/rowSums(w.dist)

# get rid of isolates
omits <- which(rowSums(w.dist)==0)

# create spatial lag of polity. round result
wmap@data$sp.lag.polity <- round((w.dist.rstd)%*%(wmap@data$polity))


# wmap dataframe is sorted on gwcode 
# get rid of isolates

map.df <- wmap@data[-omits,]
w.d.rstd <- w.dist.rstd[-omits,-omits]

# Create a SP listw object of neighbors
listw.obj <- mat2listw(w.d.rstd, style = "W")


# spatial lag
sar.fit <- lagsarlm(polity ~ lngdpvalue, data=map.df, listw.obj, type="lag",
                  method="eigen")
# spatial error
sem.fit <- errorsarlm(polity ~ lngdpvalue, data=map.df,
                      listw.obj, method="eigen", quiet=FALSE)
# OLS
ols.fit <- glm(polity ~ lngdpvalue, data=map.df)

summary(ols.fit)
summary(sar.fit)
summary(sem.fit)
###


###
# Table 5.2 

# Europe trade example

# Matrix with reverse dyads
cmat98BA <- read.table("rawdata/tradedata/dyadmat_europe98_BA-no origins 3+.csv", sep=",", header=T)

# Matrix without reverse dyads
cmat98noBA <- read.table("rawdata/tradedata/dyadmat_europe98_no BA-no origins 3+.csv", sep=",", header=T)

# Data
logdat98 <- read.table("rawdata/tradedata//dyaddata_europe98_log-no origins 3+.csv", sep=",", header=T)

# Make distance matrix a list
dlist <- vector(mode="list",length=dim(logdat98)[1])
attr(dlist,"region.id") <- logdat98$id
attr(dlist,"class") <- "nb"
ids <- unique(cmat98BA$c1)
for(i in 1:length(ids)){
  # Find their numbers in the data
  dat1 <- cmat98BA[cmat98BA$c1==ids[i],]
  dat2 <- cmat98noBA[cmat98noBA$c1==ids[i],]
  input <- sort(c(dat1$c2,dat2$c2))
  dlist[[i]] <- input
}

# OLS
tab2.ols <- glm(logtrade ~ logdem + logapop + logbpop + logargdppc + logbrgdppc + logs + logdist + logmid,
                data=logdat98,na.action=na.omit)
summary(tab2.ols)
logLik(tab2.ols)

# SEM
tab2.sem <- errorsarlm(logtrade ~ logdem + logapop + logbpop + logargdppc + logbrgdppc + logs + logdist + logmid,
                       data=logdat98,na.action=na.omit,
                       nb2listw(dlist,style="W"), method="eigen", quiet=FALSE)
summary(tab2.sem)
logLik(tab2.sem)
###

###
# Table 5.3 

# Afica trade example

# Read data
cmat98BA <- read.table("rawdata/tradedata/dyadmat_africa98_BA.csv", sep=",", header=T)
cmat98noBA <- read.table("rawdata/tradedata/dyadmat_africa98_no BA.csv", sep=",", header=T)
logdat98 <- read.table("rawdata/tradedata/dyaddata_africa98_log.csv", sep=",", header=T)

# Make weigth matrix lists
dlist <- vector(mode="list",length=dim(logdat98)[1])
attr(dlist,"region.id") <- logdat98$id
attr(dlist,"class") <- "nb"
ids <- unique(cmat98BA$c1)
for(i in 1:length(ids)){
  # Find their numbers in the data
  dat1 <- cmat98BA[cmat98BA$c1==ids[i],]
  dat2 <- cmat98noBA[cmat98noBA$c1==ids[i],]
  input <- sort(c(dat1$c2,dat2$c2))
  dlist[[i]] <- input
}


tab3.ols <- glm(logtrade ~ logdem + logapop + logbpop + logargdppc + 
                  logbrgdppc + logs + logdist + logmid,
                data=logdat98, na.action=na.omit, trace=TRUE)
summary(tab3.ols)
logLik(tab3.ols)

tab3.sem <- errorsarlm(logtrade ~ logdem + logapop + logbpop + logargdppc + 
                         logbrgdppc + logs + logdist + logmid,
                       data=logdat98, na.action=na.omit,
                       nb2listw(dlist,style="W"), method="eigen", quiet=FALSE)
summary(tab3.sem)
logLik(tab3.sem)
###

